==> leadlink_api_lc-0.0.2.tar.gz: lib/app/api_leadlink_qrcode.dart <==
import 'package:flutter/material.dart';
import 'dart:convert';
import 'api_product.dart';
import 'api_user_profile.dart';
import '../basics/api_util.dart';
import 'package:leancloud_storage/leancloud.dart';

class APILeadLinkQRCode {

  // 二维码生成字符串
  static void createQRCode(Map<String,dynamic> inputMap,{Function(String qrcodeData) onSucc,Function(String) onFail}) {

    try {
      // 转字符串
      String jsonString = json.encode(inputMap);
      // 加密
      String encryptWord = APIUtil.encrypted(jsonString);
      // 加壳
      String encryptWordShell = APIUtil.shellQRcodeString(encryptWord);
      if(onSucc != null) onSucc(encryptWordShell);
    } catch (e) {
      if(onFail != null) onFail('二维码未生成成功');
    }

  }

  // 验证是是否是力联二维码
  static bool isQRcodeAvailable(String qrcodeData) {
    return APIUtil.isAvailableQRcode(qrcodeData);
  }

  // 二维码解析
  static parseQRCode(String userId,String qrcodeData,
  {Function(LKQRCodeResult result,LCObject product,String msg) onSucc,
  Function(LKQRCodeResult result,LCObject product,String msg) onFail}) async {

    // 二维码合法性检测
    if(userId == null || userId.length == 0 || APIUtil.isAvailableQRcode(qrcodeData) == false) {
      if(onFail != null) onFail(
        LKQRCodeResult(isCanOpen: false,isForEver: false,isAuthorized: false,isRecognized: false,isUnExpired: false,
        productId: 0,expirationDate: '',productName: '',
        skuId: 0,qrcodeData: qrcodeData,isProductExist: false,authorizedState: '',
      ),null,'未检测到力联二维码,请重试');
      return;
    }

    // 脱壳
    String unShellWord = APIUtil.unshellQRcodeString(qrcodeData);
    if(unShellWord == '') {
      if(onFail != null) onFail(
        LKQRCodeResult(isCanOpen: false,isForEver: false,isAuthorized: false,isRecognized: false,isUnExpired: false,
        productId: 0,expirationDate: '',productName: '',
        skuId: 0,qrcodeData: qrcodeData,isProductExist: false,authorizedState: '',
      ),null,'未检测到力联二维码,请重试');
      return;
    }
    // 解密
    String plain = APIUtil.decrypted(unShellWord);
    if(plain == '') {
      if(onFail != null) onFail(
        LKQRCodeResult(isCanOpen: false,isForEver: false,isAuthorized: false,isRecognized: false,isUnExpired: false,
        productId: 0,expirationDate: '',productName: '',
        skuId: 0,qrcodeData: qrcodeData,isProductExist: false,authorizedState: '',
      ),null,'未检测到力联二维码,请重试');
      return;
    }

    // 解析
    try {

      // {"sk":2,"if":1,"ex":"2021-01-01","pi":1,"us":"1,2,3"}

      Map<String,dynamic> map       = json.decode(plain);
      String us                     = map['us'] ?? '';
      List users                    = us.splitMapJoin(',') ?? [];
      int isForever                 = map['if'] ?? 0 ;
      int skuId                     = map['sk'] ?? 0;
      int productId                 = map['pi'] ?? 0 ;
      String expirationDate         = map['ex'] ?? '' ;

      LKQRCodeResult result = LKQRCodeResult();
      result.qrcodeData = qrcodeData;
      result.skuId      = skuId;
      result.isRecognized = true;
      result.expirationDate = expirationDate;
      result.productId = productId;
      result.isForEver = isForever == 0 ? false : true;

      // 在有效期内判断
      if(result.isForEver == true){
        result.isUnExpired = true;
      }
      else {
        final nowDate    = DateTime.now();
        final qrcodeDate = DateTime.parse(result.expirationDate);
        result.isUnExpired = qrcodeDate.isAfter(nowDate) ? true : false ;
      }

      // 用户授权判断
      bool isHaveUser = false;
      for(int i = 0 ; i < users.length ; i++) {
        final uid = users[i];
        if(userId == uid) {
          isHaveUser = true;
          break;
        }
      }
      result.isAuthorized = isHaveUser;
      result.authorizedState = isHaveUser == true ? '已授权' : '未授权';

      // 开通按钮可用判断
      if(result.isRecognized && result.isAuthorized && result.isUnExpired) {
        result.isCanOpen = true;
      }
      else {
        result.isCanOpen = false;
      }
      
      // 获取产品详情
      APIProduct.getNewProductDetailById(productId,onFail: (_){
        result.isProductExist = false;
        if(onFail != null) onFail(result,null,'产品已下架');
      },
      onSucc: (prodcut){
        result.isProductExist = true;
        result.productName = prodcut['name'] ?? '';
        if(onSucc != null) onSucc(result,prodcut,result.isAuthorized == true ? '力联码已授权' : '力联码未授权');
      });

    } catch (e) {
      if(onFail != null) onFail(
        LKQRCodeResult(isCanOpen: false,isForEver: false,isAuthorized: false,isRecognized: false,isUnExpired: false,
        productId: 0,expirationDate: '',productName: '',
        skuId: 0,qrcodeData: qrcodeData,isProductExist: false,authorizedState: '',
      ),null,'未检测到力联二维码,请重试');
      return;
    }
  }

  // 开通方案(含上报原始二维码报文)
  static openSchemeByScanQRcode(LKQRCodeResult qrcodeResult,String schemeName,LCObject product,
    {Function(LCObject scheme) onSucc,Function(String) onFail}) async {

    try {
      
      // 获取用户信息
      APIUserProfile.currentUserProfile(onFail: onFail,onSucc: (user) async {

          // 获取产品SKU
          APIProduct.getProductSKU(qrcodeResult.skuId,onFail: onFail,onSucc: (sku) async {

              // 上报并增加方案
              LCObject object = LCObject('UserScheme');

              // 方案名称
              object['name']       = schemeName;
              object['qrcodeData'] = qrcodeResult.qrcodeData;

              // 获取当前用户
              object['user']       = user;
              object['nickName']   = user['nickName'] ?? '';
              object['mobile']     = user['mobile'] ?? '';
              object['userId']     = user['id'] ?? 0;

              // 产品
              object['product']    = product;
              object['productId']  = product[''] ?? 0;
              object['productName']= product['name'] ?? '';

              // sku
              object['sku']       = sku;
              object['skuName']   = sku['name'] ?? '';
              object['bindParam'] = sku['bindParam'] ?? {}; // 此处从接口取
              
              // 激活,设置来源
              object['isActive']  = 1;
              object['from']      = 'scan';

              final result = await object.save();
              if(onSucc != null) onSucc(result);
            });

          }
      );

    } catch (e) {
      if(onFail != null) onFail('方案开通未成功,请联系技术支持');
    }

  }


  // 测试用例
  static exampleQRCode() {


    var map = Map<String,dynamic>();
    
    map['sk'] = 2;
    map['if'] = 1;
    map['ex'] = '2021-01-01';
    map['pi'] = 1;
    map['us'] = '1,2,3';

    try {
      String jsonString = json.encode(map);
      // 原始报文
      debugPrint('加密开始==>');

      debugPrint('原始明文');
      debugPrint(jsonString);
      // 加密
      String encryptWord = APIUtil.encrypted(jsonString);
      debugPrint('加密');
      debugPrint(encryptWord);
      // 加壳
      String encryptWordShell = APIUtil.shellQRcodeString(encryptWord);
      debugPrint('加壳');
      debugPrint(encryptWordShell);

      // 解密测试
      debugPrint(' ');
      debugPrint('解密开始==>');
      debugPrint('原始二维码');
      debugPrint(encryptWordShell);

      // 判断力联码
      bool isAvailable = APIUtil.isAvailableQRcode(encryptWordShell);
      debugPrint('状态判断:' + (isAvailable == true ? '是' :'不是') + '力联二维码');

      // 脱壳
      String unShellWord = APIUtil.unshellQRcodeString(encryptWordShell);
      debugPrint('脱壳');
      debugPrint(unShellWord);

      // 解密
      String plain = APIUtil.decrypted(unShellWord);
      debugPrint('解密');
      debugPrint(plain);


    } 
    catch (e) {
      
    }
}

}

// 二维码扫描结果
class LKQRCodeResult {
  
  // 可开通
  bool isCanOpen = false;
  // 已识别成功
  bool isRecognized  = false;
  // 已授权成功
  bool isAuthorized  = false;
  // 已授权文案
  String  authorizedState;
  // 有效期范围内,未超期
  bool isUnExpired;
  // 是否是永久
  bool isForEver;
  // 有效期
  String expirationDate;
  // 产品ID
  int productId;
  // skuId
  int skuId;
  // 产品存在,产品未下架
  bool isProductExist;
  // 原始二维码
  String qrcodeData;
  // 产品名称
  String productName; 

  LKQRCodeResult({this.isCanOpen, this.isRecognized, this.isAuthorized, this.authorizedState,this.isUnExpired, this.isForEver, 
  this.expirationDate, this.productId, this.skuId, this.isProductExist,this.qrcodeData,this.productName});


}

==> leadlink_api_lc-0.0.4.tar.gz: lib/app/api_leadlink_qrcode.dart <==
import 'package:flutter/material.dart';
import 'dart:convert';
import 'api_product.dart';
import 'api_user_profile.dart';
import '../basics/api_util.dart';
import 'package:leancloud_storage/leancloud.dart';

class APILeadLinkQRCode {

  // 二维码生成字符串
  static void createQRCode(Map<String,dynamic> inputMap,{Function(String qrcodeData) onSucc,Function(String) onFail}) {

    try {
      // 转字符串
      String jsonString = json.encode(inputMap);
      // 加密
      String encryptWord = APIUtil.encrypted(jsonString);
      // 加壳
      String encryptWordShell = APIUtil.shellQRcodeString(encryptWord);
      if(onSucc != null) onSucc(encryptWordShell);
    } catch (e) {
      if(onFail != null) onFail('二维码未生成成功');
    }

  }

  // 验证是是否是力联二维码
  static bool isQRcodeAvailable(String qrcodeData) {
    return APIUtil.isAvailableQRcode(qrcodeData);
  }

  // 二维码解析
  static parseQRCode(String userId,String qrcodeData,
  {Function(LKQRCodeResult result,LCObject product,String msg) onSucc,
  Function(LKQRCodeResult result,LCObject product,String msg) onFail}) async {

    // 二维码合法性检测
    if(userId == null || userId.length == 0 || APIUtil.isAvailableQRcode(qrcodeData) == false) {
      if(onFail != null) onFail(
        LKQRCodeResult(isCanOpen: false,isForEver: false,isAuthorized: false,isRecognized: false,isUnExpired: false,
        productId: 0,expirationDate: '',productName: '',
        skuId: 0,qrcodeData: qrcodeData,isProductExist: false,authorizedState: '',
      ),null,'未检测到力联二维码,请重试');
      return;
    }

    // 脱壳
    String unShellWord = APIUtil.unshellQRcodeString(qrcodeData);
    if(unShellWord == '') {
      if(onFail != null) onFail(
        LKQRCodeResult(isCanOpen: false,isForEver: false,isAuthorized: false,isRecognized: false,isUnExpired: false,
        productId: 0,expirationDate: '',productName: '',
        skuId: 0,qrcodeData: qrcodeData,isProductExist: false,authorizedState: '',
      ),null,'未检测到力联二维码,请重试');
      return;
    }
    // 解密
    String plain = APIUtil.decrypted(unShellWord);
    if(plain == '') {
      if(onFail != null) onFail(
        LKQRCodeResult(isCanOpen: false,isForEver: false,isAuthorized: false,isRecognized: false,isUnExpired: false,
        productId: 0,expirationDate: '',productName: '',
        skuId: 0,qrcodeData: qrcodeData,isProductExist: false,authorizedState: '',
      ),null,'未检测到力联二维码,请重试');
      return;
    }

    // 解析
    try {


      Map<String,dynamic> map       = json.decode(plain);
      String us                     = map['us'] ?? '';
      List<String> users            = us.split(',') ?? [];
      int isForever                 = map['if'] ?? 0 ;
      int skuId                     = map['sk'] ?? 0;
      int productId                 = map['pi'] ?? 0 ;
      String expirationDate         = map['ex'] ?? '' ;

      LKQRCodeResult result = LKQRCodeResult();
      result.qrcodeData = qrcodeData;
      result.skuId      = skuId;
      result.isRecognized = true;
      result.expirationDate = expirationDate;
      result.productId = productId;
      result.isForEver = isForever == 0 ? false : true;

      // 在有效期内判断
      if(result.isForEver == true){
        result.isUnExpired = true;
      }
      else {
        final nowDate    = DateTime.now();
        final qrcodeDate = DateTime.parse(result.expirationDate);
        result.isUnExpired = qrcodeDate.isAfter(nowDate) ? true : false ;
      }

      // 用户授权判断
      bool isHaveUser = false;
      for(int i = 0 ; i < users.length ; i++) {
        final uid = users[i];
        if(userId == uid) {
          isHaveUser = true;
          break;
        }
      }
      result.isAuthorized = isHaveUser;
      result.authorizedState = isHaveUser == true ? '已授权' : '未授权';

      // 开通按钮可用判断
      if(result.isRecognized && result.isAuthorized && result.isUnExpired) {
        result.isCanOpen = true;
      }
      else {
        result.isCanOpen = false;
      }
      
      // 获取产品详情
      APIProduct.getNewProductDetailById(productId,onFail: (_){
        result.isProductExist = false;
        if(onFail != null) onFail(result,null,'产品已下架');
      },
      onSucc: (prodcut){
        result.isProductExist = true;
        result.productName = prodcut['name'] ?? '';
        if(onSucc != null) onSucc(result,prodcut,result.isAuthorized == true ? '力联码已授权' : '力联码未授权');
      });

    } catch (e) {
      if(onFail != null) onFail(
        LKQRCodeResult(isCanOpen: false,isForEver: false,isAuthorized: false,isRecognized: false,isUnExpired: false,
        productId: 0,expirationDate: '',productName: '',
        skuId: 0,qrcodeData: qrcodeData,isProductExist: false,authorizedState: '',
      ),null,'未检测到力联二维码,请重试');
      return;
    }
  }

  // 开通方案(含上报原始二维码报文)
  static openSchemeByScanQRcode(LKQRCodeResult qrcodeResult,String schemeName,LCObject product,
    {Function(LCObject scheme) onSucc,Function(String) onFail}) async {

    try {
      
      // 获取用户信息
      APIUserProfile.currentUserProfile(onFail: onFail,onSucc: (user) async {

          // 获取产品SKU
          APIProduct.getProductSKU(qrcodeResult.skuId,onFail: onFail,onSucc: (sku) async {

              // 上报并增加方案
              LCObject object = LCObject('UserScheme');

              // 方案名称
              object['name']       = schemeName;
              object['qrcodeData'] = qrcodeResult.qrcodeData;

              // 获取当前用户
              object['user']       = user;
              object['nickName']   = user['nickName'] ?? '';
              object['mobile']     = user['mobile'] ?? '';
              object['userId']     = user['id'] ?? 0 ;

              // 产品
              object['product']    = product;
              object['productId']  = product['id'] ?? 0 ;
              object['productName']= product['name'] ?? '';

              // sku
              object['sku']       = sku;
              object['skuName']   = sku['name'] ?? '';
              object['bindParam'] = sku['bindParam'] ?? {}; // 此处从接口取
              
              // 激活,设置来源
              object['isActive']  = 1;
              object['from']      = 'scan';

              final result = await object.save();
              if(onSucc != null) onSucc(result);
            });

          }
      );

    } catch (e) {
      if(onFail != null) onFail('方案开通未成功,请联系技术支持');
    }

  }


  // 测试用例
  static exampleQRCode() {


    var map = Map<String,dynamic>();
    
    map['sk'] = 2;
    map['if'] = 1;
    map['ex'] = '2021-01-01';
    map['pi'] = 1;
    map['us'] = '1,2,3';

    try {
      String jsonString = json.encode(map);
      // 原始报文
      debugPrint('加密开始==>');

      debugPrint('原始明文');
      debugPrint(jsonString);
      // 加密
      String encryptWord = APIUtil.encrypted(jsonString);
      debugPrint('加密');
      debugPrint(encryptWord);
      // 加壳
      String encryptWordShell = APIUtil.shellQRcodeString(encryptWord);
      debugPrint('加壳');
      debugPrint(encryptWordShell);

      // 解密测试
      debugPrint(' ');
      debugPrint('解密开始==>');
      debugPrint('原始二维码');
      debugPrint(encryptWordShell);

      // 判断力联码
      bool isAvailable = APIUtil.isAvailableQRcode(encryptWordShell);
      debugPrint('状态判断:' + (isAvailable == true ? '是' :'不是') + '力联二维码');

      // 脱壳
      String unShellWord = APIUtil.unshellQRcodeString(encryptWordShell);
      debugPrint('脱壳');
      debugPrint(unShellWord);

      // 解密
      String plain = APIUtil.decrypted(unShellWord);
      debugPrint('解密');
      debugPrint(plain);


    } 
    catch (e) {
      
    }
}

}

// 二维码扫描结果
class LKQRCodeResult {
  
  // 可开通
  bool isCanOpen = false;
  // 已识别成功
  bool isRecognized  = false;
  // 已授权成功
  bool isAuthorized  = false;
  // 已授权文案
  String  authorizedState;
  // 有效期范围内,未超期
  bool isUnExpired;
  // 是否是永久
  bool isForEver;
  // 有效期
  String expirationDate;
  // 产品ID
  int productId;
  // skuId
  int skuId;
  // 产品存在,产品未下架
  bool isProductExist;
  // 原始二维码
  String qrcodeData;
  // 产品名称
  String productName; 

  LKQRCodeResult({this.isCanOpen, this.isRecognized, this.isAuthorized, this.authorizedState,this.isUnExpired, this.isForEver, 
  this.expirationDate, this.productId, this.skuId, this.isProductExist,this.qrcodeData,this.productName});


}
